@@ -3,25 +3,30 @@ package MyApp::Service::Chat::Worker;
 use strict;
 use warnings;
 
+use MyApp::Service::Base;
 use base 'MyApp::Service::Base';
 
 
 sub on_startup {
     my $self = shift;
 
-    $self->accept_jobs(
+    $self->init_persistent_connections;
+
+    $self->accept_remote_calls(
         'myapp.chat.message'  => 'send_message',
         'myapp.chat.pmessage' => 'send_private_message',
         'myapp.chat.notice'   => 'send_notice',
         'myapp.chat.ping'     => 'ping',
     );
+
+    log_info "Ready";
 }
 
 sub send_message {
     my ($self, $params) = @_;
 
     my $msg  = $params->{'message'};
-    my $from = $self->get_current_user_uuid;
+    my $from = $self->uuid;
 
     return unless (defined $msg && length $msg);
 
@@ -45,7 +50,7 @@ sub send_private_message {
     # For simplicity, this example avoids resolving username <--> uuid 
     my $uuid = $params->{'to_user'};
     my $msg  = $params->{'message'};
-    my $from = $self->get_current_user_uuid;
+    my $from = $self->uuid;
 
     return unless (defined $msg && length $msg);
 
