@@ -6,7 +6,7 @@ use warnings;
 BEGIN { unshift @INC, ($ENV{'PERL5LIB'} =~ m/([^:]+)/g); }
 
 
-use Beekeeper::Client;
+use MyApp::Service::Flood;
 use Time::HiRes qw( time sleep );
 use Getopt::Long;
 
@@ -27,7 +27,7 @@ my $Help = "
 Usage: flood [OPTIONS]
 Flood with requests a test worker pool.
 
-  -t, --type str   type of requests to be made (N, J, B or A)
+  -t, --type str   type of requests to be made (N, S, A or F)
   -c, --count N    how many requests to be made
   -r, --rate  N    sustain a rate of N requests per second
   -s, --size  N    size in KB of requests, default is 0
@@ -35,15 +35,15 @@ Flood with requests a test worker pool.
   -b, --benchmark  run a set of predefined benchmarks
   -h, --help       display this help and exit
 
-To create a burst of 5000 notifications:
+Create a burst of 5000 notifications:
 
   flood --type N --count 5000
 
-To create a constant load of 100 jobs per second:
+Create a constant load of 100 requests per second:
 
-  flood --type J --rate 100
+  flood --type S --rate 100
 
-Run a predefined set of benchmarks
+Run a predefined set of benchmarks:
 
   flood --benchmark
 
@@ -86,47 +86,38 @@ sub time_this {
     my $payload = { data => 'X' x ($size * 1024) };
 
     my $type = $args{'type'} || 'N';
-    my @async_jobs;
     my $code;
 
     if ($type =~ m/^N(otification)?/i) {
-        $type = 'notification';
+        $type = 'notifications';
         $code = sub {
-            $client->send_notification(
-                method => 'myapp.test.flood', 
-                params => $payload,
-            );
+            # Measure send time
+            MyApp::Service::Flood->notify( $payload );
         };
     }
-    elsif ($type =~ m/^J(ob)?/i) {
-        $type = 'sync job';
+    elsif ($type =~ m/^S(ync)?/i) {
+        $type = 'sync calls';
         $code = sub {
-            $client->do_job(
-                method => 'myapp.test.echo', 
-                params => $payload,
-            );
+            # Measure response time
+            MyApp::Service::Flood->echo( $payload );
         };
     }
-    elsif ($type =~ m/^B(ackground)?(.job)?/i) {
-        $type = 'background job';
+    elsif ($type =~ m/^A(sync)?/i) {
+        $type = 'async calls';
         $code = sub {
-            $client->do_background_job(
-                method => 'myapp.test.echo', 
-                params => $payload,
-            );
+            # Measure send time only
+            MyApp::Service::Flood->async_echo( $payload, sub {} );
         };
     }
-    elsif ($type =~ m/^A(sync)?(.job)?/i) {
-        $type = 'async job';
+    elsif ($type =~ m/^F(ire)?/i) {
+        $type = 'fire & forget';
         $code = sub {
-            push @async_jobs, $client->do_async_job(
-                method => 'myapp.test.echo', 
-                params => $payload,
-            );
+            # Measure send time only
+            MyApp::Service::Flood->fire_echo( $payload );
         };
     }
     else {
-        die "type must be one of (N)otification, (J)ob, (B)ackground job or (A)sinc job\n";
+        die "type must be one of (N)otification, (S)ync, (A)sync or (F)ire and forget\n";
     }
 
     my $rate = $args{'rate'} ? (1 / $args{'rate'}) : 0;
@@ -140,7 +131,7 @@ sub time_this {
     }
 
     local $| = 1;
-    printf( "%s %-16s of %3s Kb  ", $max_count, $type.'s', $size ) if (!$rate);
+    printf( "%s %-15s of %3s Kb  ", $max_count, $type, $size ) if (!$rate);
 
     my $count = 0;
     my $start = time();
@@ -166,18 +157,15 @@ sub time_this {
         }
     }
 
-    if ($type eq 'async job') {
-        $client->wait_all_jobs;
-        @async_jobs = ();
-    }
-
     my $ellapsed = time() - $start - $sleept;
     my $took = sprintf("%.3f", $ellapsed);
     my $tps = sprintf("%.0f", $count / $ellapsed);
     my $avg = sprintf("%.2f", $ellapsed / $count * 1000);
 
-    printf( "%s %-16s of %3s Kb  ", $count, $type.'s', $size ) if ($rate);
+    printf( "%s %-15s of %3s Kb  ", $count, $type, $size ) if ($rate);
     printf( "in %6s sec  %6s /sec %6s ms each\n", $took, $tps, $avg );
+
+    $client->wait_async_calls if ($type eq 'async calls');
 }
 
 sub run_benchmarks {
@@ -194,15 +182,15 @@ sub run_benchmarks {
 
     print "\n";
 
-    # Jobs
+    # Sync calls
     foreach (@sizes) {
-        time_this( type => 'J', count => $count, size => $_ );
+        time_this( type => 'S', count => $count, size => $_ );
         sleep 1;
     }
 
     print "\n";
 
-    # Async jobs
+    # Async calls
     foreach (@sizes) {
         time_this( type => 'A', count => $count, size => $_ );
         sleep 1;
@@ -210,9 +198,9 @@ sub run_benchmarks {
 
     print "\n";
 
-    # Background jobs
+    # Fire calls
     foreach (@sizes) {
-        time_this( type => 'B', count => $count, size => $_ );
+        time_this( type => 'F', count => $count, size => $_ );
         sleep 1;
     }
 
