@@ -3,22 +3,24 @@ package MyApp::Service::Calculator;
 use strict;
 use warnings;
 
-use Beekeeper::Worker;
+use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
 
 sub on_startup {
     my $self = shift;
 
-    $self->accept_jobs(
+    $self->accept_remote_calls(
         'myapp.calculator.eval_expr' => 'eval_expr',
     );
+
+    log_info "Ready";
 }
 
 sub authorize_request {
     my ($self, $req) = @_;
 
-    return REQUEST_AUTHORIZED;
+    return BKPR_REQUEST_AUTHORIZED;
 }
 
 sub eval_expr {
