@@ -3,7 +3,7 @@ package Beekeeper;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
 1;
 
@@ -19,7 +19,7 @@ Beekeeper - Framework for building applications with a microservices architectur
  
 =head1 VERSION
  
-Version 0.03
+Version 0.06
 
 =head1 SYNOPSIS
 
@@ -69,7 +69,7 @@ Create an API for the service:
       
       my $result = $cli->call_remote(
           method => "my.service.echo",
-          params => { %args },
+          params => \%args,
       );
   
       return $result;
@@ -95,11 +95,9 @@ Beekeeper is a framework for building applications with a microservices architec
 
 =end HTML
 
-A pool of worker processes handle requests and communicate with each other through a common 
-message bus.
+A pool of worker processes handle requests and communicate with each other through a common message bus.
 
-Clients send requests through a different set of message buses, which are isolated for security 
-reasons.
+Clients send requests through a different set of message buses, which are isolated for security reasons.
 
 Requests and responses are shoveled between buses by a few router processes.
 
@@ -115,19 +113,17 @@ B<Benefits of this architecture:>
 
 B<Key characteristics:>
 
-- Broker is a MQTT messaging server, like Mosquitto or HiveMQ.
+- The broker is an MQTT messaging server, like Mosquitto, HiveMQ or EMQ X.
 
-- Broker protocol is MQTT 5 (see the specification at L<https://docs.oasis-open.org/mqtt/mqtt/v5.0/mqtt-v5.0.html>).
+- The messaging protocol is MQTT 5 (see the L<specification|https://docs.oasis-open.org/mqtt/mqtt/v5.0/mqtt-v5.0.html>).
 
-- RPC protocol is JSON-RPC 2.0 (see the specification at L<https://www.jsonrpc.org/specification>).
+- The RPC protocol is JSON-RPC 2.0 (see the L<specification|https://www.jsonrpc.org/specification>).
 
-- Message marshalling is JSON.
+- There is no message persistence in the broker, it just passes on messages.
 
-- No message persistence in the broker, it just passes on messages.
+- There is no routing logic defined in the broker.
 
-- No routing logic is defined in the broker.
-
-- Blends synchronous and asynchronous workers or clients.
+- Synchronous and asynchronous workers or clients can be integrated seamlessly.
 
 - Efficient multicast and unicast push notifications.
 
@@ -136,11 +132,11 @@ B<Key characteristics:>
 
 B<What does this framework provides:>
 
-- C<Beekeeper::Worker>, to create service workers.
+- L<Beekeeper::Worker>, to create service workers.
 
-- C<Beekeeper::Client>, to create service clients.
+- L<Beekeeper::Client>, to create service clients.
 
-- C<bkpr> command which spawns and controls worker processes.
+- L<bkpr> command which spawns and controls worker processes.
 
 - Command line tools for monitoring and controlling worker pools.
 
@@ -157,9 +153,9 @@ B<What does this framework provides:>
 
 =head3 Creating workers
 
-Workers provide a service accepting certain RPC calls from clients. The base class C<Beekeeper::Worker> 
-provides all the glue needed to accept requests and communicate trough the message bus with 
-clients or another workers.
+Workers provide a service accepting certain RPC calls from clients. The base class L<Beekeeper::Worker> 
+provides all the glue needed to accept requests and communicate trough the message bus with clients 
+or another workers.
 
 A worker class just declares on startup which methods it will accept, then implements them:
 
@@ -184,9 +180,8 @@ A worker class just declares on startup which methods it will accept, then imple
 
 =head3 Creating clients
 
-Clients of the service need an interface to use it without knowledge of the underlying RPC 
-mechanisms. The class C<Beekeeper::Client> provides methods to connect to the broker and make 
-RPC calls.
+Clients of the service need an interface to use it without knowledge of the underlying RPC mechanisms.
+The class L<Beekeeper::Client> provides methods to connect to the broker and make RPC calls.
 
 This is the interface of the above service:
 
@@ -216,13 +211,12 @@ Then other workers or clients can just:
 
 =head3 Configuring
 
-Beekeeper applications use two config files to define how clients, workers and brokers connect 
-to each other. These files are searched for in ENV C<BEEKEEPER_CONFIG_DIR>, C<~/.config/beekeeper> 
-and then C</etc/beekeeper>. File format is relaxed JSON, which allows comments and trailings 
-commas.
+Beekeeper applications use two config files to define how clients, workers and brokers connect to 
+each other. These files are looked for in ENV C<BEEKEEPER_CONFIG_DIR>, C<~/.config/beekeeper> and 
+then C</etc/beekeeper>. File format is relaxed JSON, which allows comments and trailing commas.
 
-The file C<pool.config.json> defines all worker pools running on a host, specifying which logical 
-bus should be used and which services it will run. For example:
+The file C<pool.config.json> defines all worker pools running on a host, specifying which logical bus
+should be used and which services it will run. For example:
 
   [{
       "pool_id" : "myapp",
@@ -232,8 +226,8 @@ bus should be used and which services it will run. For example:
       },
   }]
 
-The file C<bus.config.json> defines all logical buses used by the application, specifying the 
-connection parameters to the brokers that will service them. For example:
+The file C<bus.config.json> defines all logical buses used by the application, specifying the connection
+parameters to the brokers that will service them. For example:
 
   [{
       "bus_id"   : "backend",
@@ -242,100 +236,71 @@ connection parameters to the brokers that will service them. For example:
       "password" : "def456",
   }]
 
-Neither the worker code nor the client code have hardcoded references to the logical message 
-bus or the broker connection parameters, they communicate to each other using the definitions 
-in these two files.
+Neither the worker code nor the client code have hardcoded references to the logical message bus or
+the broker connection parameters, these communicate to each other using the definitions in these two files.
 
 
 =head3 Running
 
-To start or stop a pool of workers you use the C<bkpr> command. Given the above example config, 
-this will start 4 processes running C<MyApp::Worker> code:
+To start or stop a pool of workers you use the L<bkpr> command. Given the above example config, this 
+will start 4 processes running C<MyApp::Worker> code:
 
   bkpr --pool "myapp" start
 
-When started it daemonizes itself and forks all worker processes, then continues monitoring 
-those forked processes and immediately respawns defunct ones.
+When started it daemonizes itself and forks all worker processes, then continues monitoring those forked
+processes and immediately respawns defunct ones.
 
 The framework includes these command line tools to manage worker pools:
 
-- C<bkpr-top> allows to monitor in real time the performance of all workers.
+- L<bkpr-top> allows to monitor in real time the performance of workers.
 
-- C<bkpr-log> allows to monitor in real time the log output of all workers.
+- L<bkpr-log> allows to monitor in real time the log output of workers.
 
-- C<bkpr-restart> gracefully restarts local or remote worker pools.
+- L<bkpr-restart> gracefully restarts worker pools.
 
 
 =head1 Performance
 
-Beekeeper is pretty lightweight, so the performance depends mostly on *the broker* performance. 
-These are 
-ballpark performance measurements of a local setup running Mosquitto:
+Beekeeper is pretty lightweight for being pure Perl, but the performance depends mostly on *the broker*
+performance, particularly on the broker introduced latency. These are ballpark performance estimations:
 
-- A C<call_remote> synchronous call to a remote method adds 1.5 ms of latency and involves 4 network 
-round trips. This implies a maximum of 650 synchronous calls per second.
+- A C<call_remote> synchronous call involves 4 MQTT messages. A broker adds around 2.5 ms of latency 
+  processing these 4 messages, so this limits a single client to make a maximum of 400 synchronous 
+  calls per second. The cpu load will be very low (less than 1%), as the client spends most of the
+  time waiting for messages.
 
-- A C<call_remote_async> asynchronous call to a remote method takes 0.1 ms. This implies a maximum 
-of 10000 asynchronous calls per second (just the call, then it must wait for responses).
+- A C<call_remote_async> asynchronous call to a remote method takes 0.12 ms. This implies a maximum of
+  8000 asynchronous calls per second (that is just for calls, then it must wait for responses).
 
-- Scheduling a remote task with C<fire_remote> takes 0.1 ms. This implies a maximum of 
-10000 calls per second.
+- Launching a remote task with C<fire_remote> involves 1 MQTT message and takes 0.1 ms. This implies
+  a maximum of 10000 calls per second.
 
-- Sending a notification with C<send_notification> takes 0.1 ms. A worker can emit 10000 notifications 
-per second, even over 15000 if these are smaller than 1 KB.
+- Sending a notification with C<send_notification> involves 1 MQTT message and takes 0.1 ms. A worker
+  can emit more than 10000 notifications per second, even 15000 if these are smaller than 1 KB.
 
-- A worker processing remote calls adds 0.3 ms of latency and involves 2 network round trips. 
-So a single worker can handle a maximum of 3300 requests per second.
+- A worker processing remote calls can handle a maximum of 3500 requests per second. The cpu load will
+  be low for simple tasks, as the worker will spend a significant chunk of time waiting for messages.
 
-- A worker adds an overhead of 0,04% CPU load per request.
+- An empty worker uses 10 MB of resident memory for the perl interpreter and the few required modules.
+  After adding actual code to do useful work the memory usage will of course increase.
 
-- A worker uses 10 MB of resident memory.
+- A single router can handle around 4000 requests per second.
 
-- Frontend router adds 5 ms of latency and involves 2 additional network round trips.
-
-B<Hypothetical example:>
-
-Suppose it is needed to handle 1000 requests per second to a task that takes 25 ms to complete, 
-uses 20 MB of memory and has 2% CPU load. Servers are in the same datacenter and the network 
-roundtrip is 0.1 ms.
-
-Adding framework and network latency, a single worker can handle:
-
-  1000 ms / (25 ms + 0.3 ms + 0.1 ms * 2) = 39 req/s
-
-In order to handle 1000 requests per second:
-
-  1000 req/s / 39 req/s = 26 workers
-
-The memory needed is:
-
-  26 workers * (20 MB + 10 MB) = 780 MB
-
-The CPU needed is:
-
-  26 * 2% + 1000 * 0,04% = 92% = 1 core
-
-End user latency is:
-
-  25 ms + 0.3 ms + 5 ms + 0.1 ms * 6 = 31 ms + user latency
-
-Backend broker receives 2000 msg/s and sends 2000 msg/s, giving a 4000 msg/s total traffic.
-Frontend broker receives 1000 msg/s and sends 1000 msg/s, giving a 2000 msg/s total traffic.
-
-These numbers will improve a bit when running on beefier CPUs, and worsen a lot if broker performance 
-degrades under heavy load.
+- Routers add 2 ms to frontend requests roundtrip.
 
 
 =head1 Examples
 
-This distribution includes some examples that can be run out of the box using an internal C<ToyBroker> 
+This distribution includes some examples that can be run out of the box using an internal C<ToyBroker>
 (so no install of a proper broker is needed):
 
 C<examples/basic> is a barebones example of the usage of Beekeper.
 
 C<examples/flood> allows to estimate the performance of a Beekeper setup.
 
-C<examples/webstomp> use a service from a browser using WebSockets.
+C<examples/scraper> demonstrates asynchronous workers and clients.
+
+C<examples/websocket> uses a service from a browser using WebSockets.
 
 C<examples/chat> implements a real world setup with isolated buses and redundancy.
 
