@@ -3,8 +3,9 @@ package Beekeeper::Client;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
+use Beekeeper::AnyEvent;
 use Beekeeper::MQTT;
 use Beekeeper::JSONRPC;
 use Beekeeper::Config;
@@ -48,13 +49,14 @@ sub new {
     };
 
     $self->{_CLIENT} = {
-        forward_to     => $args{'forward_to'},
+        forward_to     => undef,
         response_topic => undef,
         in_progress    => undef,
         curr_request   => undef,
         caller_id      => undef,
         caller_addr    => undef,
         auth_data      => undef,
+        auth_salt      => undef,
         async_cv       => undef,
         correlation_id => 1,
         callbacks      => {},
@@ -89,6 +91,9 @@ sub new {
         }
     }
 
+    $self->{_CLIENT}->{forward_to} = delete $args{'forward_to'};
+    $self->{_CLIENT}->{auth_salt}  = delete $args{'auth_salt'};
+
     # Start a fresh new MQTT session on connect
     $args{'clean_start'} = 1;
 
@@ -138,7 +143,7 @@ sub send_notification {
 
     $fq_meth =~ m/^     ( [\w-]+ (?:\.[\w-]+)* )
                      \. ( [\w-]+ ) 
-                 (?: \@ ( [\w-]+ ) (\.[\w-]+)* )? $/x or croak "Invalid method $fq_meth";
+                 (?: \@ ( [\w-]+ ) (\.[\w-]+)* )? $/x or croak "Invalid method '$fq_meth'";
 
     my ($service, $method, $remote_bus, $addr) = ($1, $2, $3, $4);
 
@@ -182,22 +187,25 @@ sub send_notification {
 sub accept_notifications {
     my ($self, %args) = @_;
 
+    my ($file, $line) = (caller)[1,2];
+    my $at = "at $file line $line\n";
+
     my $callbacks = $self->{_CLIENT}->{callbacks};
 
     foreach my $fq_meth (keys %args) {
 
         $fq_meth =~ m/^  ( [\w-]+ (?: \.[\w-]+ )* ) 
-                      \. ( [\w-]+ | \* ) $/x or croak "Invalid notification method $fq_meth";
+                      \. ( [\w-]+ | \* ) $/x or croak "Invalid notification method '$fq_meth'";
 
         my ($service, $method) = ($1, $2);
 
         my $callback = $args{$fq_meth};
 
         unless (ref $callback eq 'CODE') {
-            croak "Invalid callback for '$method'";
+            croak "Invalid callback for '$fq_meth'";
         }
 
-        croak "Already accepting notifications $fq_meth" if exists $callbacks->{"msg.$fq_meth"};
+        croak "Already accepting notifications '$fq_meth'" if exists $callbacks->{"msg.$fq_meth"};
         $callbacks->{"msg.$fq_meth"} = $callback;
 
         #TODO: Allow to accept private notifications without subscribing
@@ -210,23 +218,23 @@ sub accept_notifications {
         $self->{_BUS}->subscribe(
             topic      => $topic,
             on_publish => sub {
-                my ($body_ref, $msg_headers) = @_;
+                my ($payload_ref, $mqtt_properties) = @_;
 
                 local $@;
-                my $request = eval { decode_json($$body_ref) };
+                my $request = eval { decode_json($$payload_ref) };
 
                 unless (ref $request eq 'HASH' && $request->{jsonrpc} eq '2.0') {
-                    warn "Received invalid JSON-RPC 2.0 notification";
+                    warn "Received invalid JSON-RPC 2.0 notification $at";
                     return;
                 }
 
                 bless $request, 'Beekeeper::JSONRPC::Notification';
-                $request->{_headers} = $msg_headers;
+                $request->{_mqtt_properties} = $mqtt_properties;
 
                 my $method = $request->{method};
 
                 unless (defined $method && $method =~ m/^([\.\w-]+)\.([\w-]+)$/) {
-                    warn "Received notification with invalid method $method";
+                    warn "Received notification with invalid method '$method' $at";
                     return;
                 }
 
@@ -234,7 +242,7 @@ sub accept_notifications {
                          $callbacks->{"msg.$1.*"};
 
                 unless ($cb) {
-                    warn "No callback found for received notification $method";
+                    warn "No callback found for received notification '$method' $at";
                     return;
                 }
 
@@ -242,7 +250,7 @@ sub accept_notifications {
             },
             on_suback => sub {
                 my ($success, $prop) = @_;
-                croak "Could not subscribe to $topic" unless $success;
+                die "Could not subscribe to topic '$topic' $at" unless $success;
             }
         );
     }
@@ -252,17 +260,20 @@ sub accept_notifications {
 sub stop_accepting_notifications {
     my ($self, @methods) = @_;
 
+    my ($file, $line) = (caller)[1,2];
+    my $at = "at $file line $line\n";
+
     croak "No method specified" unless @methods;
 
     foreach my $fq_meth (@methods) {
 
         $fq_meth =~ m/^  ( [\w-]+ (?: \.[\w-]+ )* ) 
-                      \. ( [\w-]+ | \* ) $/x or croak "Invalid method $fq_meth";
+                      \. ( [\w-]+ | \* ) $/x or croak "Invalid method '$fq_meth'";
 
         my ($service, $method) = ($1, $2);
 
         unless (defined $self->{_CLIENT}->{callbacks}->{"msg.$fq_meth"}) {
-            carp "Not previously accepting notifications $fq_meth";
+            carp "Not previously accepting notifications '$fq_meth'";
             next;
         }
 
@@ -274,20 +285,11 @@ sub stop_accepting_notifications {
         $self->{_BUS}->unsubscribe(
             topic       => $topic,
             on_unsuback => sub {
+                my ($success, $prop) = @_;
+
+                die "Could not unsubscribe from topic '$topic' $at" unless $success; 
 
                 delete $self->{_CLIENT}->{callbacks}->{"msg.$fq_meth"};
-
-                # Discard notifications already queued
-                my $job_queue = $self->{_WORKER}->{job_queue_high};
-
-                @$job_queue = grep {
-                    my $task = $_;
-                    my ($body_ref, $msg_headers) = @$task;
-                    my $request = decode_json($$body_ref);
-                    my $req_method = $request->{method};
-                    $req_method =~ m/^([\.\w-]+)\.([\w-]+)$/;
-                    not ($service eq $1 && ($method eq '*' || $method eq $2));
-                } @$job_queue;
             },
         );
     }
@@ -303,7 +305,7 @@ sub call_remote {
 
     # Make AnyEvent allow one level of recursive condvar blocking, as we may
     # block both in $worker->__work_forever and in $client->__do_rpc_request
-    $AE_WAITING && croak "Recursive condvar blocking wait attempted";
+    $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
     local $AE_WAITING = 1;
     local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -324,15 +326,15 @@ sub call_remote_async {
     my $self = shift;
 
     my $req = $self->__do_rpc_request( @_, req_type => 'ASYNCHRONOUS' );
-    
+
     return $req;
 }
 
 sub fire_remote {
     my $self = shift;
 
-    # Send job to a worker, but do not wait for result
-    $self->__do_rpc_request( @_, req_type => 'BACKGROUND' );
+    # Send request to a worker, but do not wait for response
+    $self->__do_rpc_request( @_, req_type => 'FIRE_FORGET' );
 
     return;
 }
@@ -349,7 +351,7 @@ sub __do_rpc_request {
 
     $fq_meth =~ m/^     ( [\w-]+ (?:\.[\w-]+)* )
                      \. ( [\w-]+ ) 
-                 (?: \@ ( [\w-]+ ) (\.[\w-]+)* )? $/x or croak "Invalid method $fq_meth";
+                 (?: \@ ( [\w-]+ ) (\.[\w-]+)* )? $/x or croak "Invalid method '$fq_meth'";
 
     my ($service, $method, $remote_bus, $addr) = ($1, $2, $3, $4);
 
@@ -379,7 +381,7 @@ sub __do_rpc_request {
     $send_args{'auth'} = $client->{auth_data} if defined $client->{auth_data};
     $send_args{'clid'} = $client->{caller_id} if defined $client->{caller_id};
 
-    my $BACKGROUND  = $args{req_type} eq 'BACKGROUND';
+    my $FIRE_FORGET = $args{req_type} eq 'FIRE_FORGET';
     my $SYNCHRONOUS = $args{req_type} eq 'SYNCHRONOUS';
     my $raise_error = $args{'raise_error'};
     my $req_id;
@@ -395,7 +397,7 @@ sub __do_rpc_request {
     $send_args{'response_topic'} = $client->{response_topic} ||
                                    $self->__create_response_topic;
 
-    unless ($BACKGROUND) {
+    unless ($FIRE_FORGET) {
         # Assign an unique request id (unique only for this client)
         $req_id = $client->{correlation_id}++;
         $req->{'id'} = $req_id;
@@ -414,7 +416,7 @@ sub __do_rpc_request {
         %send_args,
     );
 
-    if ($BACKGROUND) {
+    if ($FIRE_FORGET) {
          # Nothing else to do
          return;
     }
@@ -469,6 +471,9 @@ sub __create_response_topic {
     my $self = shift;
     my $client = $self->{_CLIENT};
 
+    my ($file, $line) = (caller(2))[1,2];
+    my $at = "at $file line $line\n";
+
     # Subscribe to an exclusive topic for receiving RPC responses
 
     my $response_topic = 'priv/' . $self->{_BUS}->{client_id};
@@ -478,13 +483,13 @@ sub __create_response_topic {
         topic       => $response_topic,
         maximum_qos => 0,
         on_publish  => sub {
-            my ($body_ref, $msg_headers) = @_;
+            my ($payload_ref, $mqtt_properties) = @_;
 
             local $@;
-            my $resp = eval { decode_json($$body_ref) };
+            my $resp = eval { decode_json($$payload_ref) };
 
             unless (ref $resp eq 'HASH' && $resp->{jsonrpc} eq '2.0') {
-                warn "Received invalid JSON-RPC 2.0 message";
+                warn "Received invalid JSON-RPC 2.0 message $at";
                 return;
             }
 
@@ -519,12 +524,12 @@ sub __create_response_topic {
                 # Unicasted notification
 
                 bless $resp, 'Beekeeper::JSONRPC::Notification';
-                $resp->{_headers} = $msg_headers;
+                $resp->{_headers} = $mqtt_properties;
 
                 my $method = $resp->{method};
 
                 unless (defined $method && $method =~ m/^([\.\w-]+)\.([\w-]+)$/) {
-                    warn "Received notification with invalid method $method";
+                    warn "Received notification with invalid method '$method' $at";
                     return;
                 }
 
@@ -532,7 +537,7 @@ sub __create_response_topic {
                          $client->{callbacks}->{"msg.$1.*"};
 
                 unless ($cb) {
-                    warn "No callback found for received notification $method";
+                    warn "No callback found for received notification '$method' $at";
                     return;
                 }
 
@@ -541,7 +546,7 @@ sub __create_response_topic {
         },
         on_suback => sub {
             my ($success, $prop) = @_;
-            croak "Could not subscribe to $response_topic" unless $success;
+            die "Could not subscribe to response topic '$response_topic' $at" unless $success;
         }
     );
 
@@ -557,7 +562,7 @@ sub wait_async_calls {
 
     # Make AnyEvent to allow one level of recursive condvar blocking, as we may
     # block both in $worker->__work_forever and here
-    $AE_WAITING && croak "Recursive condvar blocking wait attempted";
+    $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
     local $AE_WAITING = 1;
     local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -580,20 +585,25 @@ sub set_authentication_data {
 sub __use_authorization_token {
     my ($self, $token) = @_;
 
-    my $secret = 'salt'; #TODO: read from config file
+    # Using a hashing function makes harder to access the wrong worker pool by mistake,
+    # but it is not an effective access restriction: anyone with access to the backend
+    # bus credentials can easily inspect and clone auth data tokens
+
+    my $salt = $self->{_CLIENT}->{auth_salt} || '';
 
     my $adata_ref = \$self->{_CLIENT}->{auth_data};
 
     my $guard = Beekeeper::Client::Guard->new( $adata_ref );
 
-    $$adata_ref = sha256_hex($token . $secret);
+    $$adata_ref = sha256_hex($token . $salt);
 
     return $guard;
 }
 
 1;
 
-package Beekeeper::Client::Guard;
+package
+    Beekeeper::Client::Guard;   # hide from PAUSE
 
 sub new {
     my ($class, $ref) = @_;
@@ -620,7 +630,7 @@ Beekeeper::Client - Make RPC calls through message bus
 
 =head1 VERSION
  
-Version 0.03
+Version 0.06
 
 =head1 SYNOPSIS
 
@@ -678,8 +688,8 @@ All methods in this module are exported by default to C<Beekeeper::Worker>.
 
 Connects to the message broker and returns a singleton instance.
 
-Unless explicit connection parameters to the broker are provided tries 
-to connect using the configuration from config file C<bus.config.json>.
+Unless explicit connection parameters to the broker are provided it tries 
+to connect using the parameters defined in config file C<bus.config.json>.
 
 =head1 METHODS
 
@@ -687,10 +697,11 @@ to connect using the configuration from config file C<bus.config.json>.
 
 Broadcast a notification to the message bus.
 
-All clients and workers listening for C<method> will receive it. If no one is listening
-the notification is lost.
+All clients and workers listening for given method will receive it. 
 
-=over 4
+If no one is listening for it the notification will be discarded.
+
+=over
 
 =item method
 
@@ -719,7 +730,7 @@ C<$callback> is a coderef that will be called when a notification is received.
 When executed, the callback will receive a parameter C<$params> which contains
 the notification value or data structure sent.
 
-Note that callbacks will not be executed timely if AnyEvent loop is not running.
+Please note that callbacks will not be executed timely if AnyEvent loop is not running.
 
 =head3 stop_accepting_notifications ( $method, ... )
 
@@ -738,13 +749,13 @@ C<$method> must be one of the strings used previously in C<accept_notifications>
 Makes a synchronous RPC call to a service worker through the message bus.
 
 It will wait (in the event loop) until a response is received, wich will be either
-an C<Beekeeper::JSONRPC::Response> object or a C<Beekeeper::JSONRPC::Error>.
+a L<Beekeeper::JSONRPC::Response> object or a L<Beekeeper::JSONRPC::Error>.
 
 On error it will die unless C<raise_error> option is set to false.
 
 This method accepts the following parameters:
 
-=over 4
+=over
 
 =item method
 
@@ -768,9 +779,9 @@ request takes too long but otherwise was executed successfully the response will
 eventually arrive but it will be ignored.
 
 =item raise_error
- 
+
 If set to true (the default) dies with the received error message when a call returns
-an error response. If set to false returns a C<Beekeeper::JSONRPC::Error> instead.
+an error response. If set to false returns a L<Beekeeper::JSONRPC::Error> instead.
 
 =back
 
@@ -778,44 +789,46 @@ an error response. If set to false returns a C<Beekeeper::JSONRPC::Error> instea
 
 Makes an asynchronous RPC call to a service worker through the message bus.
 
-It returns immediately a C<Beekeeper::JSONRPC::Request> object which, once completed,
+It returns immediately a L<Beekeeper::JSONRPC::Request> object which, once completed,
 will have a defined C<response>.
 
 This method  accepts parameters C<method>, C<params>, C<address> and C<timeout> 
 the same as C<call_remote>. Additionally two callbacks can be specified:
 
-=over 4
+=over
 
 =item on_success
 
 Callback which will be executed after receiving a successful response with a
-C<Beekeeper::JSONRPC::Response> object as parameter. Must be a coderef.
+L<Beekeeper::JSONRPC::Response> object as parameter. Must be a coderef.
 
 =item on_error
 
 Callback which will be executed after receiving an error response with a
-C<Beekeeper::JSONRPC::Error> object as parameter. Must be a coderef.
+L<Beekeeper::JSONRPC::Error> object as parameter. Must be a coderef.
 
 =back
 
 =head3 fire_remote ( %args )
 
-Makes an asynchronous RPC call to a service worker through the message bus but
-does not expect to receive any response, it is a fire and forget call.
+Fire and forget an RPC call to a service worker through the message bus.
 
-It returns undef immediately.
+It returns undef immediately. The worker receiving the call will not send back a response.
 
-This method  accepts parameters C<method>, C<params>, C<address> and C<timeout> 
-the same as C<call_remote>.
+This method accepts parameters C<method>, C<params> and C<address> the same as C<call_remote>.
 
 =head3 wait_async_calls
 
-Wait (in the event loop) until all calls made by C<call_remote_async> are completed.
+Wait (running the event loop) until all calls made by C<call_remote_async> are completed
+either by success, error or timeout.
 
 =head3 set_authentication_data ( $data )
 
-Add an arbitrary authentication data blob to subsequent jobs requests or 
-notifications sent.
+Add an arbitrary authentication data blob to subsequent calls or notifications sent.
+
+This data persists for client lifetime in standalone clients. Within worker context
+it persists until the end of current request only, and will be piggybacked on
+calls made to another workers within the scope of current request.
 
 The meaning of this data is application specific, this framework doesn't give 
 any special one to it.
@@ -826,7 +839,7 @@ Gets the current authentication data blob.
 
 =head1 SEE ALSO
  
-L<Beekeeper::MQTT>, L<Beekeeper::Worker>.
+L<Beekeeper::Worker>, L<Beekeeper::MQTT>.
 
 =head1 AUTHOR
 
