@@ -3,10 +3,11 @@ package Beekeeper::MQTT;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
 use AnyEvent;
 use AnyEvent::Handle;
+use Time::HiRes;
 use List::Util 'shuffle';
 use Scalar::Util 'weaken';
 use Exporter 'import';
@@ -261,6 +262,8 @@ sub _fatal {
     $self->{error_cb}->($errstr);
 }
 
+our $BUSY_SINCE = undef;
+our $BUSY_TIME  = 0;
 
 sub connect {
     my ($self, %args) = @_;
@@ -366,11 +369,19 @@ sub _connect {
             my $offs;
             my $byte;
 
+            my $timing_packets;
+
+            unless (defined $BUSY_SINCE) {
+                # Measure time elapsed while processing incoming packets
+                $BUSY_SINCE = Time::HiRes::time;
+                $timing_packets = 1; 
+            }
+
             PARSE_PACKET: {
 
                 $rbuff_len = length $fh->{rbuf};
 
-                return unless $rbuff_len >= 2;
+                last PARSE_PACKET unless $rbuff_len >= 2;
 
                 unless ($packet_type) {
 
@@ -382,7 +393,7 @@ sub _connect {
                         $byte = unpack "C", substr( $fh->{rbuf}, $offs++, 1 );
                         $packet_len += ($byte & 0x7f) * $mult;
                         last unless ($byte & 0x80);
-                        return if ($offs >= $rbuff_len); # Not enough data
+                        last PARSE_PACKET if ($offs >= $rbuff_len); # Not enough data
                         $mult *= 128;
                         redo if ($offs < 5);
                     }
@@ -396,7 +407,7 @@ sub _connect {
 
                 if ($rbuff_len < ($offs + $packet_len)) {
                     # Not enough data
-                    return;
+                    last PARSE_PACKET;
                 }
 
                 # Consume packet from buffer
@@ -464,6 +475,11 @@ sub _connect {
                 # Handle could have been destroyed at this point
                 redo PARSE_PACKET if defined $fh->{rbuf};
             }
+
+            if (defined $timing_packets) {
+                $BUSY_TIME += Time::HiRes::time - $BUSY_SINCE;
+                undef $BUSY_SINCE;
+            }
         },
     );
 
@@ -925,7 +941,7 @@ sub subscribe {
     }
 
     my $packet_id = $self->{packet_seq}++;
-    $self->{packet_seq} = 1 if $packet_id > 0xFFFF;
+    $self->{packet_seq} = 1 if $packet_id == 0xFFFF;
 
     # Set callback for incomings PUBLISH
     my $subscr_id = $self->{subscr_seq}++;
@@ -1076,7 +1092,7 @@ sub unsubscribe {
     }
 
     my $packet_id = $self->{packet_seq}++;
-    $self->{packet_seq} = 1 if $packet_id > 0xFFFF;
+    $self->{packet_seq} = 1 if $packet_id == 0xFFFF;
 
     # Set callback for UNSUBACK
     $self->{packet_cb}->{$packet_id} = {
@@ -1242,7 +1258,7 @@ sub publish {
     my $packet_id;
     if ($qos) {
         $packet_id = $self->{packet_seq}++;
-        $self->{packet_seq} = 1 if $packet_id > 0xFFFF;
+        $self->{packet_seq} = 1 if $packet_id == 0xFFFF;
     }
 
     my $raw_prop = '';
@@ -1337,7 +1353,7 @@ sub publish {
         # As client may be syncronous, wait until entire message is sent.
 
         # Make AnyEvent allow one level of recursive condvar blocking
-        $AE_WAITING && croak "Recursive condvar blocking wait attempted";
+        $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
         local $AE_WAITING = 1;
         local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -1717,7 +1733,7 @@ sub flush_buffer {
         # Kernel write buffer is full, see publish() above
 
         # Make AnyEvent allow one level of recursive condvar blocking
-        $AE_WAITING && croak "Recursive condvar blocking wait attempted";
+        $AE_WAITING && Carp::confess "Recursive condvar blocking wait attempted";
         local $AE_WAITING = 1;
         local $AnyEvent::CondVar::Base::WAITING = 0;
 
@@ -1764,11 +1780,11 @@ __END__
 
 =head1 NAME
  
-Beekeeper::MQTT - A lightweight asynchronous MQTT 5.0 client.
+Beekeeper::MQTT - Asynchronous MQTT 5.0 client.
  
 =head1 VERSION
  
-Version 0.03
+Version 0.06
 
 =head1 SYNOPSIS
 
@@ -1805,7 +1821,7 @@ Version 0.03
   
   $mqtt->disconnect;
 
-Most methods allows to send arbitrary properties along with commands.
+Most methods allow to send arbitrary properties as key-value pairs of utf8 strings.
 
 Except for trivial cases, error checking is delegated to the server.
 
@@ -1819,7 +1835,7 @@ The MQTT specification can be found at L<https://mqtt.org/mqtt-specification>
 
 =head3 new ( %options )
 
-=over 4
+=over
 
 =item host
 
@@ -1829,7 +1845,7 @@ a randomly choosen node of the cluster.
 
 =item port
 
-Port of the MQTT server. If not specified use the MQTT default of 1818.
+Port of the MQTT server. If not specified use the MQTT default of 1883.
 
 =item tls
 
@@ -1862,7 +1878,7 @@ connection when this is called.
 
 Connect to the MQTT server and do handshake. On failure retries until timeout.
 
-=over 4
+=over
 
 =item blocking => $bool
 
@@ -1877,12 +1893,9 @@ Callback which is executed after the server accepted the connection.
 
 =head3 disconnect ( %args )
 
-A client can disconnect from the server at anytime by closing the socket but 
-there is no guarantee that the previously sent packets have been received by
-the server. This method should be called to do a graceful shutdown, where the
-client is assured that all previous packets have been received by the server.
+Does a graceful disconnection from the server.
 
-=over 4
+=over
 
 =item $reason_code
 
@@ -1905,7 +1918,7 @@ it will be passed to given on_publish callback:
       },
   );
 
-=over 4
+=over
 
 =item topics => \@topics
 
@@ -1937,7 +1950,7 @@ from that topic. Example:
       topics => ['topic/foo','topic/bar']
   );
 
-=over 4
+=over
 
 =item topics => \@topics
 
@@ -1963,7 +1976,7 @@ Sends a message to a topic. Example:
       }
   );
 
-=over 4
+=over
 
 =item topic => $str
 
@@ -1984,7 +1997,7 @@ Must be set to a true value to indicate a message retransmission.
 
 =item retain => $bool
 
-...
+When true sets the message retain flag.
 
 =item message_expiry_interval => $int
 
@@ -1993,7 +2006,7 @@ period has ellapsed.
 
 =item response_topic => $str
 
-...
+Utf8 string containing the response topic name.
 
 =item on_puback => $cb->($reason_code)
 
@@ -2012,7 +2025,7 @@ of utf8 strings.
 Used to acknowledge the receipt of a message received from a subscription with QoS 1.
 The server should resend the message until it is acknowledged.
 
-=over 4
+=over
 
 =item packet_id => $int
 
