@@ -3,9 +3,8 @@ package Beekeeper::Service::Router::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
-use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -143,7 +142,7 @@ sub on_shutdown {
     my $tmr = AnyEvent->timer( after => 30, cb => sub { $cv->send });
     $cv->recv;
 
-    # 4. Just in case of pool stop, wait for workers to finish their current jobs
+    # 4. Just in case of pool full stop, wait for workers to finish their current tasks
     my $wait = AnyEvent->condvar;
     $tmr = AnyEvent->timer( after => SHUTDOWN_WAIT, cb => sub { $wait->send });
     $wait->recv;
@@ -206,7 +205,7 @@ sub pull_frontend_requests {
 
         my $src_queue = "\$share/BKPR/req/$backend_role-$lane";
 
-        my ($payload_ref, $msg_prop);
+        my ($payload_ref, $mqtt_properties);
         my ($dest_queue, $reply_to, $caller_id, $mqtt_session);
         my %pub_args;
 
@@ -214,16 +213,16 @@ sub pull_frontend_requests {
             topic       => $src_queue,
             maximum_qos => 0,
             on_publish  => sub {
-                ($payload_ref, $msg_prop) = @_;
+                ($payload_ref, $mqtt_properties) = @_;
 
                 # (!) UNTRUSTED REQUEST
 
                 # eg: req/backend/myapp/service
-                $dest_queue = $msg_prop->{'fwd_to'} || '';
+                $dest_queue = $mqtt_properties->{'fwd_to'} || '';
                 return unless $dest_queue =~ m|^req(/(?!_)[\w-]+)+$|;
 
                 # eg: priv/7nXDsxMDwgLUSedX
-                $reply_to = $msg_prop->{'response_topic'} || '';
+                $reply_to = $mqtt_properties->{'response_topic'} || '';
                 return unless $reply_to =~ m|^priv/(\w{16,23})$|;
                 $caller_id = $1;
 
@@ -249,7 +248,7 @@ sub pull_frontend_requests {
 
                 DEBUG && log_trace "Forwarded request:  $src_queue \@$frontend_id --> $dest_queue \@$backend_id";
 
-                $self->{_WORKER}->{jobs_count}++;
+                $self->{_WORKER}->{calls_count}++;
             },
             on_suback => sub {
                 log_debug "Forwarding $src_queue \@$frontend_id --> req/$backend_role/{app}/{service} \@$backend_id";
@@ -276,15 +275,15 @@ sub pull_backend_responses {
 
         my $src_queue = "\$share/BKPR/res/$frontend_id-$lane";
 
-        my ($payload_ref, $msg_prop, $dest_queue);
+        my ($payload_ref, $mqtt_properties, $dest_queue);
 
         $backend_bus->subscribe(
             topic       => $src_queue,
             maximum_qos => 0,
             on_publish  => sub {
-                ($payload_ref, $msg_prop) = @_;
+                ($payload_ref, $mqtt_properties) = @_;
 
-                ($dest_queue) = split('@', $msg_prop->{'addr'}, 2);
+                ($dest_queue) = split('@', $mqtt_properties->{'addr'}, 2);
 
                 $frontend_bus->publish(
                     topic   => $dest_queue,
@@ -328,15 +327,15 @@ sub pull_backend_notifications {
 
         my $src_queue = "\$share/BKPR/msg/$frontend_role-$lane",
 
-        my ($payload_ref, $msg_prop, $destination, $address);
+        my ($payload_ref, $mqtt_properties, $destination, $address);
 
         $backend_bus->subscribe(
             topic       => $src_queue,
             maximum_qos => 0,
             on_publish  => sub {
-                ($payload_ref, $msg_prop) = @_;
+                ($payload_ref, $mqtt_properties) = @_;
 
-                ($destination, $address) = split('@', $msg_prop->{'fwd_to'}, 2);
+                ($destination, $address) = split('@', $mqtt_properties->{'fwd_to'}, 2);
 
                 if (defined $address) {
 
@@ -404,6 +403,8 @@ sub _init_routing_table {
                 my $addr  = $value->[0];
                 my $topic = $value->[1];
 
+                return unless defined $addr;
+
                 my $relpy_topics = $self->{Addr_to_topics}->{$addr} ||= [];
                 return if grep { $_ eq $topic } @$relpy_topics;
                 push @$relpy_topics, $topic;
@@ -416,6 +417,8 @@ sub _init_routing_table {
                 my $addr  = $old_value->[0];
                 my $topic = $old_value->[1];
 
+                return unless defined $addr;
+
                 my $relpy_topics = $self->{Addr_to_topics}->{$addr} || return;
                 @$relpy_topics = grep { $_ ne $topic } @$relpy_topics;
                 delete $self->{Addr_to_topics}->{$addr} unless @$relpy_topics;
@@ -428,12 +431,12 @@ sub _init_routing_table {
     );
 
     $self->accept_remote_calls(
-        '_bkpr.router.assign_addr' => 'assign_address',
-        '_bkpr.router.remove_addr' => 'remove_address',
+        '_bkpr.router.bind'   => 'bind_remote_session',
+        '_bkpr.router.unbind' => 'unbind_remote_session',
     );
 }
 
-sub assign_address {
+sub bind_remote_session {
     my ($self, $params) = @_;
 
     my $address     = $params->{address};
@@ -441,13 +444,6 @@ sub assign_address {
     my $caller_addr = $params->{caller_addr};
     my $auth_data   = $params->{auth_data};
 
-    my $frontend_role = $self->{frontend_role};
-
-    unless (defined $address && $address =~ m/^[\w-]+\.[\w-]+$/) {
-        # eg: frontend.user-1234
-        die ( $address ? "Invalid address $address" : "address not specified");
-    }
-
     unless (defined $caller_id && $caller_id =~ m/^\w{16,}$/) {
         # eg: 7nXDsxMDwgLUSedX
         die ( $caller_id ? "Invalid caller_id $caller_id" : "caller_id not specified");
@@ -458,19 +454,29 @@ sub assign_address {
         die ( $caller_id ? "Invalid caller_addr $caller_addr" : "caller_addr not specified");
     }
 
-    unless ( $address =~ m/^$frontend_role\./) {
-        # eg: frontend.user-1234
-        die ( "Invalid address $address: router can handle only $frontend_role.* namespace" );
-    }
+    if (defined $address) {
 
-    $address =~ s/^$frontend_role\.//;
+        my $frontend_role = $self->{frontend_role};
+
+        unless ($address =~ m/^[\w-]+\.[\w-]+$/) {
+            # eg: frontend.user-1234
+            die ( "Invalid address $address" );
+        }
+
+        unless ($address =~ m/^$frontend_role\./) {
+            # eg: frontend.user-1234
+            die ( "Invalid address $address: router can handle only $frontend_role.* namespace" );
+        }
+
+        $address =~ s/^$frontend_role\.//;
+    }
 
     $self->{MqttSessions}->set( $caller_id => [ $address, $caller_addr, $auth_data ] );
 
     return 1;
 }
 
-sub remove_address {
+sub unbind_remote_session {
     my ($self, $params) = @_;
 
     my $caller_id = $params->{caller_id};
@@ -525,23 +531,35 @@ __END__
 
 =head1 NAME
  
-Beekeeper::Service::Router::Worker - Route messages between backend and frontend
+Beekeeper::Service::Router::Worker - Route messages between backend and frontend buses
 
 =head1 VERSION
  
-Version 0.03
+Version 0.06
 
 =head1 SYNOPSIS
 
 =head1 DESCRIPTION
 
-This worker pulls requests from any frontend brokers and forward them to the 
-single backend broker it is connected to. It also pull generated responses from
-the backend and forward them to the aproppiate frontend broker which the
-client is connected to.
+Router workers pull requests from all frontend brokers and forward them to the single
+backend broker it is connected to, and pull generated responses from the backend and
+forward them to the aproppiate frontend broker which the client is connected to.
 
-In order to push unicasted notifications it keeps a shared table of client
-connections and server side assigned arbitrary addresses.
+Additionally, routers include some primitives that can be used to implement session
+management and push notifications. In order to push unicasted notifications, routers will
+keep an in-memory shared table of client connections and server side assigned addresses.
+
+If the application does not bind client sessions the routers can scale really well,
+as you can have a lot of them in a large number of servers. 
+
+But please note that when the application does use the session binding mechanism all
+routers will need the in-memory shared table, and this shared table will not scale as 
+well as the rest of the system. So a better strategy (some kind of partition) will 
+be needed for applications with a large number of concurrent clients.
+
+=head1 SEE ALSO
+ 
+L<Beekeeper::Service::Router>
 
 =head1 AUTHOR
 
