@@ -3,9 +3,8 @@ package Beekeeper::Service::Sinkhole::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
-use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -21,7 +20,7 @@ sub authorize_request {
         return unless $self->__has_authorization_token('BKPR_SYSTEM');
     }
 
-    # All requests will be rejected by reject_job 
+    # All requests will be rejected later by reject_request 
     return BKPR_REQUEST_AUTHORIZED;
 }
 
@@ -71,7 +70,7 @@ sub on_unserviced_queues {
         my $local_bus = $self->{_BUS}->{bus_role};
         log_error "Draining unserviced req/$local_bus/$queue";
 
-        $self->accept_remote_calls( "$queue.*" => 'reject_job' );
+        $self->accept_remote_calls( "$queue.*" => 'reject_request' );
     }
 }
 
@@ -98,12 +97,12 @@ sub on_worker_status {
     }
 }
 
-sub reject_job {
+sub reject_request {
     my ($self, $params, $req) = @_;
 
     # Just return a JSONRPC error response
 
-    if ($req->get_auth_tokens) {
+    if ($req->mqtt_properties->{'auth'}) {
         # When client provided some kind of authentication tell him the truth
         # about the service being down. Otherwise the one trying to fix the 
         # issue may be deceived into looking for auth/permissions problems
@@ -124,11 +123,11 @@ __END__
 
 =head1 NAME
  
-Beekeeper::Service::Sinkhole::Worker - Handle unserviced job queues
+Beekeeper::Service::Sinkhole::Worker - Handle unserviced call topics
 
 =head1 VERSION
  
-Version 0.03
+Version 0.06
 
 =head1 DESCRIPTION
 
@@ -138,15 +137,17 @@ disruption in the application, as any other service depending of the broken
 one will halt too for the duration of the timeout.
 
 In order to mitigate this situation all Sinkhole workers will be notified by
-the Supervisor when unserviced queues are detected, making these to respond 
+the Supervisor when unserviced topics are detected, making these to respond 
 immediately to all requests with an error response. Then callers will quickly 
 receive an error response instead of timing out.
 
 As soon as a worker of the downed service becomes online again the Sinkhole
 workers will stop rejecting requests.
 
-A Sinkhole worker is created automatically in every worker pool, and it can 
-handle around 500 req/s. Extra workers can simply be declared into config file.
+Sinkhole workers are not created automatically. In order to add Sinkhole workers
+to a pool these must be declared into config file C<pool.config.json>.
+
+A single Sinkhole worker can handle around 3500 req/s.
 
 =head1 AUTHOR
 
