@@ -3,9 +3,8 @@ package Beekeeper::Service::Supervisor::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
-use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -276,7 +275,7 @@ sub get_services_status {
 
     foreach my $worker (@$workers) {
         $services{$worker->{class}}{count}++;
-        $services{$worker->{class}}{jps}  += $worker->{jps};
+        $services{$worker->{class}}{cps}  += $worker->{cps};
         $services{$worker->{class}}{nps}  += $worker->{nps};
         $services{$worker->{class}}{cpu}  += $worker->{cpu} || 0;
         $services{$worker->{class}}{mem}  += $worker->{mem} || 0;
@@ -288,7 +287,7 @@ sub get_services_status {
     }
 
     foreach my $service (values %services) {
-        $service->{jps}  = sprintf("%.2f", $service->{jps} );
+        $service->{cps}  = sprintf("%.2f", $service->{cps} );
         $service->{nps}  = sprintf("%.2f", $service->{nps} );
         $service->{cpu}  = sprintf("%.2f", $service->{cpu} );
         $service->{mem}  = sprintf("%.2f", $service->{mem} );
@@ -412,7 +411,7 @@ Beekeeper::Service::Supervisor::Worker - Worker pool supervisor.
 
 =head1 VERSION
 
-Version 0.03
+Version 0.06
 
 =head1 DESCRIPTION
 
@@ -424,20 +423,20 @@ workers periodic performance reports.
 
 =head3 worker_status
 
-Handler for 'supervisor.worker_status' job.
+Handler for 'supervisor.worker_status' request.
 
-This job is sent by workers every few seconds and acts as a heart-beat.
+This request is made by workers every few seconds and acts as a heart-beat.
 It contains statistical data about worker performance.
 
-Note that workers doing long jobs (like slow SQL queries) may not send 
+Note that workers processing long tasks (like slow SQL queries) may not make 
 this request timely.
 
 =head3 on_worker_exit
 
-Handler for 'supervisor.worker_exit' job.
+Handler for 'supervisor.worker_exit' request.
 
-This job is sent by workers just before exiting gracefully. It is not sent 
-when worker is terminated abruptly (as process has no chance to do so).
+This request is made by workers just before exiting gracefully. It is not made 
+when a worker is terminated abruptly (as the process has no chance to do so).
 
 =head3 check_workers
 
@@ -462,13 +461,13 @@ error response instead of timing out.
 
 =head3 get_workers_status
 
-Handler for 'supervisor.get_workers_status' job.
+Handler for 'supervisor.get_workers_status' request.
 
 Used by bkpr-top command line tool.
 
 =head3 get_services_status
 
-Handler for 'supervisor.get_services_status' job.
+Handler for 'supervisor.get_services_status' request.
 
 Used by bkpr-top command line tool.
 
