@@ -3,9 +3,8 @@ package Beekeeper::Service::ToyBroker::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
-use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -107,7 +106,7 @@ sub start_listener {
     my ($self, $listener) = @_;
     weaken($self);
 
-    my $max_packet_size = $listener->{'max_packet_size'} || 65536;
+    my $max_packet_size = $listener->{'max_packet_size'};
 
     my $addr = $listener->{'listen_addr'} || '127.0.0.1';  # Must be an IPv4 or IPv6 address
     my $port = $listener->{'listen_port'} ||  1883;
@@ -157,7 +156,7 @@ sub start_listener {
                             redo if ($offs < 5);
                         }
 
-                        if ($packet_len > $max_packet_size) {
+                        if ($max_packet_size && $packet_len > $max_packet_size) {
                             $self->disconnect($fh, reason_code => 0x95);
                             return;
                         }
@@ -1265,7 +1264,8 @@ sub unsubscribe_client {
 }
 
 
-package Beekeeper::Service::ToyBroker::Client;
+package
+    Beekeeper::Service::ToyBroker::Client;   # hide from PAUSE
 
 sub new {
     my ($class, %args) = @_;
@@ -1331,7 +1331,8 @@ sub resend_unacked_messages {
 }
 
 
-package Beekeeper::Service::ToyBroker::TopicFilter;
+package
+    Beekeeper::Service::ToyBroker::TopicFilter;   # hide from PAUSE
 
 sub new {
     my ($class, $topic_filter) = @_;
@@ -1397,7 +1398,8 @@ sub remove_shared_subscription {
 }
 
 
-package Beekeeper::Service::ToyBroker::Subscription;
+package
+    Beekeeper::Service::ToyBroker::Subscription;   # hide from PAUSE
 
 sub new {
     my $class = shift;
@@ -1436,7 +1438,8 @@ sub send_message {
 }
 
 
-package Beekeeper::Service::ToyBroker::SharedSubscription;
+package
+    Beekeeper::Service::ToyBroker::SharedSubscription;   # hide from PAUSE
 
 sub new {
     my ($class, %args) = @_;
@@ -1505,15 +1508,40 @@ Beekeeper::Service::ToyBroker::Worker - Basic MQTT 5 broker
 
 =head1 VERSION
 
-Version 0.03
+Version 0.06
 
 =head1 DESCRIPTION
 
 ToyBroker implements a small MQTT subset needed to run a Beekeeper worker pool.
 
-Being single threaded it does not scale at all, but it is handy for development
+Being single threaded it does not scale at all, but it is suitable for development
 or running tests.
 
+ToyBroker is configured from file C<toybroker.config.json>, which is looked for 
+in ENV C<BEEKEEPER_CONFIG_DIR>, C<~/.config/beekeeper> and then C</etc/beekeeper>.
+
+Example configuration:
+
+  [
+      {
+          "listen_addr" : "127.0.0.1",
+          "listen_port" : "1883",
+  
+          "users" : {
+              "backend" : { "password" : "def456" },
+          },
+      },
+      {
+          "listen_addr" : "127.0.0.1",
+          "listen_port" : "8001",
+  
+          "users" : {
+              "frontend" : { "password" : "abc123" },
+              "router"   : { "password" : "ghi789" },
+          },
+      },
+  ]
+
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
