@@ -45,12 +45,12 @@ sub test_01_notifications : Test(2) {
     is( $var, $expected, "Catchall notifications received by 2 workers");
 }
 
-sub test_02_sync_jobs : Test(20) {
+sub test_02_sync_calls : Test(20) {
     my $self = shift;
     
     my $cli = Beekeeper::Client->instance;
 
-    my $resp = $cli->do_job(
+    my $resp = $cli->call_remote(
         method => 'test.echo',
         params => "foo",
     );
@@ -59,7 +59,7 @@ sub test_02_sync_jobs : Test(20) {
     is( $resp->success, 1 );
     is( $resp->result, 'foo');
 
-    $resp = $cli->do_job(
+    $resp = $cli->call_remote(
         method => 'test.echo',
         params => [ 1, { a => 2 }, "" ],
     );
@@ -68,7 +68,7 @@ sub test_02_sync_jobs : Test(20) {
     is_deeply( $resp->result, [ 1, { a => 2 }, "" ]);
 
     # Unhandled exception with no raise_error
-    $resp = $cli->do_job(
+    $resp = $cli->call_remote(
         method => 'test.fail',
         params => { 'die' => "error message 123" },
         raise_error => 0,
@@ -82,7 +82,7 @@ sub test_02_sync_jobs : Test(20) {
 
     # Unhandled exception dies
     $resp = eval {
-        $cli->do_job(
+        $cli->call_remote(
             method => 'test.fail',
             params => { 'die' => "error message 456" },
         );
@@ -93,7 +93,7 @@ sub test_02_sync_jobs : Test(20) {
 
     # Handled exception
     $resp = eval {
-        $cli->do_job(
+        $cli->call_remote(
             method => 'test.fail',
             params => { 'error' => "error message 678" },
         );
@@ -104,17 +104,17 @@ sub test_02_sync_jobs : Test(20) {
 
     # Invalid method
     $resp = eval {
-        $cli->do_job(
-            method  => 'test.#@@@@',
+        $cli->call_remote(
+            method  => 'test.#',
         );
     };
 
     is( $resp, undef );
-    like( $@, qr/Invalid method test.#@@@@ at /); # local error, call not made
+    like( $@, qr/Invalid method 'test.#' at /); # local error, call not made
 
     # Invalid method
     $resp = eval {
-        $cli->do_job(
+        $cli->call_remote(
             method  => 'test.notfound',
         );
     };
@@ -124,7 +124,7 @@ sub test_02_sync_jobs : Test(20) {
 
     # Timeout
     $resp = eval {
-        $cli->do_job(
+        $cli->call_remote(
             method  => 'test.sleep',
             params  => '0.1',
             timeout => '0.01',
@@ -135,7 +135,7 @@ sub test_02_sync_jobs : Test(20) {
     like( $@, qr/Call to 'test.sleep' failed: -31600 Request timeout at /);
 }
 
-sub test_03_background_jobs : Test(1) {
+sub test_03_fire_and_forget_calls : Test(1) {
     my $self = shift;
 
     my $cli = Beekeeper::Client->instance;
@@ -145,7 +145,7 @@ sub test_03_background_jobs : Test(1) {
 
     foreach my $n (1..3) {
 
-        $cli->do_background_job(
+        $cli->fire_remote(
             method => "test.signal",
             params => { signal => 'USR1', pid => $$, after => $n/2 },
         );
@@ -153,15 +153,15 @@ sub test_03_background_jobs : Test(1) {
 
     my $expected = 597;
     my $max_wait = 10; while ($max_wait--) { sleep 0.5; last if $var == $expected }
-    is( $var, $expected, "Background job executed 3 times");
+    is( $var, $expected, "Fire and forget method executed 3 times");
 }
 
-sub test_04_async_jobs : Test(18) {
+sub test_04_async_calls : Test(25) {
     my $self = shift;
 
     my $cli = Beekeeper::Client->instance;
 
-    my $req = $cli->do_async_job(
+    my $req = $cli->call_remote_async(
         method => 'test.echo',
         params => "baz",
     );
@@ -171,7 +171,7 @@ sub test_04_async_jobs : Test(18) {
     is( $req->success, undef );
     is( $req->result, undef );
 
-    $cli->wait_all_jobs;
+    $cli->wait_async_calls;
 
     isa_ok( $req->response, 'Beekeeper::JSONRPC::Response' );
     is( $req->success, 1 );
@@ -182,34 +182,107 @@ sub test_04_async_jobs : Test(18) {
     my $var = 239;
 
     foreach my $n (1..$count) {
-        my $req = $cli->do_async_job(
+        my $req = $cli->call_remote_async(
             method => 'test.echo',
             params => $var + $n,
         );
         push @reqs, $req;
     }
 
-    $cli->wait_all_jobs;
+    $cli->wait_async_calls;
 
     foreach my $n (1..$count) {
         is( $reqs[$n-1]->result, $var + $n );
     }
 
+    # on_success callback
+    my $cb = AnyEvent->condvar;
+    my $resp;
+
+    $cli->call_remote_async(
+        method => 'test.echo',
+        params => 749,
+        on_success => sub {
+            $resp = shift;
+            $cb->send;
+        }
+    );
+
+    $cb->recv;
+    isa_ok( $resp, 'Beekeeper::JSONRPC::Response' );
+    is( $resp->success, 1 );
+    is( $resp->result, 749 );
+
+    # on_error callback
+    $cb = AnyEvent->condvar;
+    undef $resp;
+
+    $cli->call_remote_async(
+        method => 'test.fail',
+        params => { error => "error message 821" },
+        on_error => sub {
+            $resp = shift;
+            $cb->send;
+        }
+    );
+
+    $cb->recv;
+    isa_ok( $resp, 'Beekeeper::JSONRPC::Error' );
+    is( $resp->success, 0 );
+    is( $resp->code, -32000);
+    is( $resp->message, "error message 821");
+
     # Timeout
     eval {
-        my $req = $cli->do_job(
+        my $req = $cli->call_remote(
             method  => 'test.sleep',
             params  => '0.2',
             timeout => '0.01',
         );
 
-        $cli->wait_all_jobs;
+        $cli->wait_async_calls;
     };
 
     like( $@, qr/Call to 'test.sleep' failed: -31600 Request timeout /);
 }
 
-sub test_05_client_api : Test(8) {
+sub test_05_utf8_serialization : Test(10) {
+    my $self = shift;
+
+    my $utf8_string = "\x{263A}";
+
+    my $binary_blob = $utf8_string;
+    utf8::encode($binary_blob);
+
+    is( length($utf8_string), 1, 'String length is 1 char' );
+    is( length($binary_blob), 3, 'Blob length is 3 bytes' );
+
+    my $cli = Beekeeper::Client->instance;
+
+    my $resp = $cli->call_remote(
+        method => 'test.echo',
+        params => {
+            'utf8_string' => $utf8_string,
+            'binary_blob' => $binary_blob,
+        },
+    );
+
+    isa_ok($resp, 'Beekeeper::JSONRPC::Response');
+    is( $resp->success, 1 );
+
+    my $result = $resp->result;
+
+    ok( $result->{utf8_string}, 'Received string' );
+    ok( $result->{binary_blob}, 'Received blob' );
+
+    is( length($result->{utf8_string}), 1, 'Received string length is 1 char' );
+    is( length($result->{binary_blob}), 3, 'Received blob length is 3 bytes' );
+
+    is( $result->{utf8_string}, $utf8_string, "Got correct utf8 string");
+    is( $result->{binary_blob}, $binary_blob, "Got correct binary blob");
+}
+
+sub test_06_client_api : Test(8) {
     my $self = shift;
 
     use_ok('Tests::Service::Client');
@@ -233,7 +306,7 @@ sub test_05_client_api : Test(8) {
     is( $resp->result, 'foo');
 
 
-    $resp = $svc->fibonacci_1( 1 );
+    $resp = $svc->fibonacci( 2 );
 
     isa_ok($resp, 'Beekeeper::JSONRPC::Response');
     is( $resp->success, 1 );
