@@ -14,10 +14,10 @@ use constant DEBUG => 0;
 sub test_01_term_signal : Test(21) {
     my $self = shift;
 
-    ## Test that broker complete all jobs when workers are stopped with TERM
+    ## Test that broker complete all requests before quitting when workers are stopped with TERM
 
     if ($self->automated_testing) {
-        # It is hard to make this test run reliably on smoke testers platforms
+        # This test does not run reliably on some smoke testers constrained platforms
         return "This test may fail when not enough system resources are available";
     }
 
@@ -29,15 +29,15 @@ sub test_01_term_signal : Test(21) {
     for (1..20) {
 
         for (1..8) {
-            # Give them more work than they can do, to ensure that the job queue is full
-            push @req, $cli->do_async_job(
+            # Give them more work than they can do, to ensure that the task queue is full
+            push @req, $cli->call_remote_async(
                 method  => 'test.sleep',
                 params  => .5,
                 timeout => 30,
             );
         }
 
-        # When workers are stopped they probably hold a queued job that must be processed 
+        # When workers are stopped they are probably processing a request that must be completed before quitting
         my $old = shift @worker_pids;
         DEBUG && diag "Stopping TERM worker $old";
         kill('TERM', $old);
@@ -50,10 +50,10 @@ sub test_01_term_signal : Test(21) {
     }
 
     DEBUG && diag "Waiting for backlog";
-    $cli->wait_all_jobs;
+    $cli->wait_async_calls;
 
     my @ok = grep { $_->success } @req;
-    is( scalar(@ok), scalar(@req), "All jobs executed " . scalar(@ok). "/". scalar(@req));
+    is( scalar(@ok), scalar(@req), "All calls executed " . scalar(@ok). "/". scalar(@req));
 }
 
 1;
